@@ -1,6 +1,6 @@
 /* Kernel crypto API AF_ALG interface code
  *
- * Copyright (C) 2015 - 2017, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2015 - 2018, Stephan Mueller <smueller@chronox.de>
  *
  * License: see COPYING file in root directory
  *
@@ -30,6 +30,7 @@
 #include <time.h>
 #include <sys/select.h>
 #include <sys/utsname.h>
+#include <sys/prctl.h>
 
 #include "cryptouser.h"
 #include "kcapi.h"
@@ -283,7 +284,7 @@ int32_t _kcapi_common_vmsplice_iov_fd(struct kcapi_handle *handle, int *fdptr,
 		    ret);
 
 	if ((uint32_t)ret != inlen) {
-		kcapi_dolog(KCAPI_LOG_ERR, "vmsplice: not all data received by kernel (data recieved: %ld -- data sent: %lu)",
+		kcapi_dolog(KCAPI_LOG_ERR, "vmsplice: not all data received by kernel (data received: %ld -- data sent: %lu)",
 			(long)ret, (unsigned long)inlen);
 		return -EFAULT;
 	}
@@ -358,11 +359,9 @@ int32_t _kcapi_common_vmsplice_chunk_fd(struct kcapi_handle *handle, int *fdptr,
 }
 
 /* Wrapper for io_getevents -- returns < 0 on error, or processed bytes */
-int32_t _kcapi_aio_read_all(struct kcapi_handle *handle, uint32_t toread,
-			    struct timespec *timeout)
+int _kcapi_aio_read_all(struct kcapi_handle *handle, uint32_t toread,
+			struct timespec *timeout)
 {
-	int32_t processed = 0;
-
 	if (toread > KCAPI_AIO_CONCURRENT)
 		return -EINVAL;
 
@@ -382,8 +381,11 @@ int32_t _kcapi_aio_read_all(struct kcapi_handle *handle, uint32_t toread,
 			 * If one cipher operation fails, so will the entire
 			 * AIO operation
 			 */
-			if (events[i].res < 0)
-				return events[i].res;
+			if (events[i].res < 0) {
+				handle->aio.iocb_ret[events[i].data] =
+							events[i].res;
+				return (int)events[i].res;
+			}
 
 			cb = (struct iocb *)(uintptr_t)events[i].obj;
 
@@ -391,51 +393,20 @@ int32_t _kcapi_aio_read_all(struct kcapi_handle *handle, uint32_t toread,
 			 * Older symmetric AIO implementations used a wrong
 			 * return code.
 			 */
-			if (events[i].res > 0)
-				processed = processed + events[i].res;
-			else
-				processed = processed + cb->aio_nbytes;
+			if (events[i].res > 0) {
+				handle->aio.iocb_ret[events[i].data] =
+								events[i].res;
+			} else {
+				handle->aio.iocb_ret[events[i].data] =
+								cb->aio_nbytes;
+			}
 
 			cb->aio_fildes = 0;
-			handle->aio.completed_reads++;
 		}
 		toread -= rc;
 	}
 
-	return processed;
-}
-
-/* read data from successfully processed cipher operations */
-static int _kcapi_aio_poll_data(struct kcapi_handle *handle, struct timeval *tv)
-{
-	struct timespec timeout;
-	fd_set rfds;
-	u_int64_t eval = 0;
-	int ret;
-	int efd = handle->aio.efd;
-
-	FD_ZERO(&rfds);
-	FD_SET(efd, &rfds);
-
-	ret = select(efd + 1, &rfds, NULL, NULL, tv);
-	if (ret == -1) {
-		kcapi_dolog(KCAPI_LOG_ERR, "Select Error: %d\n", errno);
-		return -errno;
-	}
-	if (!FD_ISSET(efd, &rfds)) {
-		kcapi_dolog(KCAPI_LOG_ERR, "aio poll: no FDs\n");
-		return -EFAULT;
-	}
-
-	if (read(efd, &eval, sizeof(eval)) != sizeof(eval)) {
-		kcapi_dolog(KCAPI_LOG_ERR, "efd read error\n");
-		return -EFAULT;
-	}
-
-	timeout.tv_sec = 0;
-	timeout.tv_nsec = 0;
-
-	return _kcapi_aio_read_all(handle, eval, &timeout);
+	return 0;
 }
 
 int _kcapi_aio_send_iov(struct kcapi_handle *handle, struct iovec *iov,
@@ -466,8 +437,8 @@ int _kcapi_aio_send_iov(struct kcapi_handle *handle, struct iovec *iov,
 	return 0;
 }
 
-int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
-			       struct iovec *iov, uint32_t iovlen)
+int _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
+			   struct iovec *iov, uint32_t iovlen)
 {
 	struct iocb *cb = handle->aio.cio;
 	uint32_t i;
@@ -478,11 +449,11 @@ int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
 
 	for (i = 0; i < iovlen; i++) {
 		while (cb->aio_fildes) {
-			struct timeval tv;
+			struct timespec timeout;
 
-			tv.tv_sec = 0;
-			tv.tv_usec = 10;
-			ret = _kcapi_aio_poll_data(handle, &tv);
+			timeout.tv_sec = 0;
+			timeout.tv_nsec = 10000;
+			ret = _kcapi_aio_read_all(handle, iovlen, &timeout);
 			if (ret < 0)
 				return ret;
 		}
@@ -496,6 +467,9 @@ int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
 		cb->aio_nbytes = iov->iov_len;
 		cb->aio_flags = IOCB_FLAG_RESFD;
 		cb->aio_resfd = handle->aio.efd;
+
+		handle->aio.iocb_ret[i] = AIO_OUTSTANDING;
+
 		cb++;
 		iov++;
 	}
@@ -513,7 +487,7 @@ int32_t _kcapi_aio_read_iov_fd(struct kcapi_handle *handle, int *fdptr,
 		}
 	}
 
-	return _kcapi_aio_poll_data(handle, NULL);
+	return _kcapi_aio_read_all(handle, iovlen, NULL);
 }
 
 int32_t _kcapi_common_recv_data_fd(struct kcapi_handle *handle, int *fdptr,
@@ -592,8 +566,8 @@ int _kcapi_common_setkey(struct kcapi_handle *handle,
 	ret = setsockopt(handle->tfmfd, SOL_ALG, ALG_SET_KEY, key, keylen);
 	if (ret < 0)
 		ret = -errno;
-	kcapi_dolog(KCAPI_LOG_DEBUG, "AF_ALG: sendmsg syscall returned %d",
-		    ret);
+	kcapi_dolog(KCAPI_LOG_DEBUG,
+		    "AF_ALG setkey: setsockopt syscall returned %d", ret);
 
 	return ret;
 }
@@ -841,17 +815,21 @@ static int _kcapi_common_getinfo(struct kcapi_handle *handle,
 
 static inline void _kcapi_aio_destroy(struct kcapi_handle *handle)
 {
-	if (handle->aio.disable == true)
-		return;
 	if (handle->aio.efd != -1)
 		close(handle->aio.efd);
-	io_destroy(handle->aio.aio_ctx);
+	handle->aio.efd = -1;
+	if (handle->aio.aio_ctx)
+		io_destroy(handle->aio.aio_ctx);
+	handle->aio.aio_ctx = 0;
 	if (handle->aio.cio)
 		free(handle->aio.cio);
 	handle->aio.cio = NULL;
 	if (handle->aio.ciopp)
 		free(handle->aio.ciopp);
 	handle->aio.ciopp = NULL;
+	if (handle->aio.iocb_ret)
+		free(handle->aio.iocb_ret);
+	handle->aio.iocb_ret = NULL;
 }
 
 void _kcapi_handle_destroy_nofree(struct kcapi_handle *handle)
@@ -957,6 +935,12 @@ static int _kcapi_aio_init(struct kcapi_handle *handle, const char *type)
 		goto err;
 	}
 
+	handle->aio.iocb_ret = calloc(KCAPI_AIO_CONCURRENT, sizeof(__s64));
+	if (!handle->aio.iocb_ret) {
+		err = -ENOMEM;
+		goto err;
+	}
+
 	/*
 	 * Set up the pointers to pointers array that is required by
 	 * io_submit. Please do not ask me why the kernel wants this. :-)
@@ -985,16 +969,8 @@ static int _kcapi_aio_init(struct kcapi_handle *handle, const char *type)
 	return 0;
 
 err:
-	handle->aio.disable = true;
-	if (handle->aio.efd != -1)
-		close(handle->aio.efd);
-	handle->aio.efd = -1;
-	if (handle->aio.cio)
-		free(handle->aio.cio);
-	handle->aio.cio = NULL;
-	if (handle->aio.ciopp)
-		free(handle->aio.ciopp);
-	handle->aio.ciopp = NULL;
+	_kcapi_aio_destroy(handle);
+
 	return err;
 }
 
@@ -1194,21 +1170,19 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 				struct iovec *iniov, struct iovec *outiov,
 				uint32_t iovlen, int access, int enc)
 {
-	int32_t processed = 0;
+	uint32_t i, outstanding = 0, processed = 0, inflight = 0;
 	int32_t ret;
-	uint32_t tosend = iovlen;
 
 	if (handle->aio.disable == true) {
 		kcapi_dolog(KCAPI_LOG_WARN, "AIO support disabled\n");
 		return -EOPNOTSUPP;
 	}
 
-	handle->aio.completed_reads = 0;
-
 	/* Every IOVEC is processed as its individual cipher operation. */
-	while (tosend) {
-		uint32_t process = (KCAPI_AIO_CONCURRENT < tosend) ?
-					KCAPI_AIO_CONCURRENT : tosend;
+	while (iovlen) {
+		uint32_t max_process = KCAPI_AIO_CONCURRENT - inflight;
+		uint32_t process = (max_process < iovlen) ?
+							max_process : iovlen;
 
 		ret = _kcapi_aio_send_iov(handle, iniov, process, access, enc);
 		if (ret < 0)
@@ -1220,8 +1194,24 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 
 		iniov += process;
 		outiov += process;
-		processed += ret;
-		tosend = iovlen - handle->aio.completed_reads;
+		iovlen -= process;
+
+		inflight = 0;
+		for (i = 0; i < KCAPI_AIO_CONCURRENT; i++) {
+			if (handle->aio.iocb_ret[i] == AIO_OUTSTANDING) {
+				inflight++;
+			} else if (handle->aio.iocb_ret[i] < 0) {
+				return handle->aio.iocb_ret[i];
+			} else {
+				if (handle->aio.iocb_ret[i] > INT_MAX)
+					return -EOVERFLOW;
+
+				processed += (uint32_t)handle->aio.iocb_ret[i];
+				if (processed > INT_MAX)
+					return -EOVERFLOW;
+				handle->aio.iocb_ret[i] = 0;
+			}
+		}
 	}
 
 	/*
@@ -1236,12 +1226,37 @@ int32_t _kcapi_cipher_crypt_aio(struct kcapi_handle *handle,
 	 * able to detect which particular request is completed. Thus, an
 	 * "open-ended" multi-staged AIO operation could not be implemented.
 	 */
+	for (i = 0; i < KCAPI_AIO_CONCURRENT; i++) {
+		if (handle->aio.iocb_ret[i] == AIO_OUTSTANDING)
+			outstanding++;
+	}
 
-	ret = _kcapi_aio_read_all(handle, iovlen - handle->aio.completed_reads,
-				  NULL);
+	ret = _kcapi_aio_read_all(handle, outstanding, NULL);
 	if (ret < 0)
 		return ret;
-	processed += ret;
 
+	for (i = 0; i < KCAPI_AIO_CONCURRENT; i++) {
+		if (handle->aio.iocb_ret[i] == AIO_OUTSTANDING) {
+			return -EBADMSG;
+		} else if (handle->aio.iocb_ret[i] < 0) {
+			return handle->aio.iocb_ret[i];
+		} else {
+			if (handle->aio.iocb_ret[i] > INT_MAX)
+				return -EOVERFLOW;
+
+			processed += (uint32_t)handle->aio.iocb_ret[i];
+			if (processed > INT_MAX)
+				return -EOVERFLOW;
+			handle->aio.iocb_ret[i] = 0;
+		}
+	}
 	return processed;
 }
+
+void __attribute__ ((constructor)) kcapi_library_init(void)
+{
+	/* Disable the dumping of this process as we handle with keys */
+	if (prctl(PR_SET_DUMPABLE, 0) < 0)
+		kcapi_dolog(KCAPI_LOG_ERR,
+			    "Cannot set prctl(PR_SET_DUMPABLE)");
+}
