@@ -24,7 +24,8 @@
  * AEAD ciphers
  ****************************************************************************/
 
-static int cp_aead_init_test(struct cp_test *test, size_t len, int enc, int ccm)
+static int cp_aead_init_test(struct cp_test *test, size_t len, int enc, int ccm,
+			     unsigned int aio)
 {
 	unsigned char *input = NULL;
 	unsigned char *output = NULL;
@@ -43,7 +44,8 @@ static int cp_aead_init_test(struct cp_test *test, size_t len, int enc, int ccm)
 		return -EFAULT;
 	}
 
-	if (kcapi_aead_init(&test->u.aead.handle, test->driver_name, 0)) {
+	if (kcapi_aead_init(&test->u.aead.handle, test->driver_name,
+			    aio ? KCAPI_INIT_AIO : 0)) {
 		printf(DRIVER_NAME": could not allocate aead handle for "
 		       "%s\n", test->driver_name);
 		goto out;
@@ -96,13 +98,13 @@ static int cp_aead_init_test(struct cp_test *test, size_t len, int enc, int ccm)
 	}
 
 	if (posix_memalign((void *)&input, sysconf(_SC_PAGESIZE),
-			   test->u.aead.indatalen)) {
+			   test->u.aead.indatalen * (aio ? aio : 1))) {
 		printf(DRIVER_NAME": could not allocate input buffer for "
 		       "%s\n", test->driver_name);
 		goto out;
 	}
 	if (posix_memalign((void *)&output, sysconf(_SC_PAGESIZE),
-			   test->u.aead.outdatalen)) {
+			   test->u.aead.outdatalen * (aio ? aio : 1))) {
 		printf(DRIVER_NAME": could not allocate output buffer for "
 		       "%s\n", test->driver_name);
 		goto out;
@@ -147,6 +149,75 @@ static int cp_aead_init_test(struct cp_test *test, size_t len, int enc, int ccm)
 		}
 	}
 
+	if (aio) {
+		unsigned int i;
+
+		if (posix_memalign((void *)&test->u.aead.iniov, BLOCKLEN,
+				     aio * sizeof(struct iovec))) {
+			printf(DRIVER_NAME": could not allocate iniov buffer\n");
+			goto out;
+		}
+		if (posix_memalign((void *)&test->u.aead.outiov, BLOCKLEN,
+				     aio * sizeof(struct iovec))) {
+			free(test->u.aead.iniov);
+			printf(DRIVER_NAME": could not allocate outiov buffer\n");
+			goto out;
+		}
+
+		for (i = 0; i < aio; i++) {
+			int ret = 0;
+
+			test->u.aead.iniov[i].iov_base = input;
+			test->u.aead.iniov[i].iov_len = test->u.aead.indatalen;
+			test->u.aead.outiov[i].iov_base = output;
+			test->u.aead.outiov[i].iov_len = test->u.aead.outdatalen;
+
+			if (!i) {
+				input += test->u.aead.indatalen;
+				output += test->u.aead.outdatalen;
+				continue;
+			}
+
+			if (enc) {
+				cp_read_random(input, test->u.aead.indatalen);
+			} else {
+				/* we need good data to avoid testing just the hash */
+				cp_read_random(output, test->u.aead.outdatalen);
+				ret = kcapi_aead_encrypt(test->u.aead.handle,
+							output,
+							test->u.aead.outdatalen,
+							test->u.aead.iv,
+							input,
+							test->u.aead.indatalen, 0);
+				if (ret < 0) {
+					printf(DRIVER_NAME": could not create ciphertext for "
+				"%s (%d)\n", test->driver_name, ret);
+					goto out;
+				}
+				/* copy the AAD as this is not copied by the kernel */
+				memcpy(input, output, test->u.aead.assoclen);
+				/* just a verification to avoid being fooled */
+				ret = kcapi_aead_decrypt(test->u.aead.handle,
+							input,
+							test->u.aead.indatalen,
+							test->u.aead.iv,
+							output,
+							test->u.aead.outdatalen,
+							test->accesstype);
+				if (ret < 0) {
+					printf(DRIVER_NAME": could not decrypt ciphertext for "
+				"%s (%d)\n", test->driver_name, ret);
+					goto out;
+				}
+			}
+
+			input += test->u.aead.indatalen;
+			output += test->u.aead.outdatalen;
+		}
+
+		test->u.aead.aio = aio;
+	}
+
 	return 0;
 
 out:
@@ -160,24 +231,28 @@ out:
 	return -ENOMEM;
 }
 
-static int cp_aead_init_enc_dflt(struct cp_test *test, size_t len)
+static int cp_aead_init_enc_dflt(struct cp_test *test, size_t len,
+				 unsigned int aio)
 {
-	return cp_aead_init_test(test, len, 1, 0);
+	return cp_aead_init_test(test, len, 1, 0, aio);
 }
 
-static int cp_aead_init_enc_ccm(struct cp_test *test, size_t len)
+static int cp_aead_init_enc_ccm(struct cp_test *test, size_t len,
+				unsigned int aio)
 {
-	return cp_aead_init_test(test, len, 1, 1);
+	return cp_aead_init_test(test, len, 1, 1, aio);
 }
 
-static int cp_aead_init_dec_dflt(struct cp_test *test, size_t len)
+static int cp_aead_init_dec_dflt(struct cp_test *test, size_t len,
+				 unsigned int aio)
 {
-	return cp_aead_init_test(test, len, 0, 0);
+	return cp_aead_init_test(test, len, 0, 0, aio);
 }
 
-static int cp_aead_init_dec_ccm(struct cp_test *test, size_t len)
+static int cp_aead_init_dec_ccm(struct cp_test *test, size_t len,
+				unsigned int aio)
 {
-	return cp_aead_init_test(test, len, 0, 1);
+	return cp_aead_init_test(test, len, 0, 1, aio);
 }
 
 static void cp_aead_fini_test(struct cp_test *test)
@@ -186,30 +261,50 @@ static void cp_aead_fini_test(struct cp_test *test)
 	free(test->u.aead.input);
 	free(test->u.aead.output);
 	free(test->u.aead.iv);
+	if (test->u.aead.aio) {
+		free(test->u.aead.iniov);
+		free(test->u.aead.outiov);
+	}
 	kcapi_cipher_destroy(test->u.aead.handle);
 }
 
 static unsigned int cp_ablkcipher_enc_test(struct cp_test *test)
 {
-	kcapi_aead_encrypt(test->u.aead.handle,
-			   test->u.aead.input,
-			   test->u.aead.indatalen,
-			   test->u.aead.iv,
-			   test->u.aead.output,
-			   test->u.aead.outdatalen,
-			   test->accesstype);
+	if (test->u.aead.aio)
+		kcapi_aead_encrypt_aio(test->u.aead.handle,
+				       test->u.aead.iniov,
+				       test->u.aead.outiov,
+				       test->u.aead.aio,
+				       test->u.aead.iv,
+				       test->accesstype);
+	else
+		kcapi_aead_encrypt(test->u.aead.handle,
+				   test->u.aead.input,
+				   test->u.aead.indatalen,
+				   test->u.aead.iv,
+				   test->u.aead.output,
+				   test->u.aead.outdatalen,
+				   test->accesstype);
 	return test->u.aead.outdatalen;
 }
 
 static unsigned int cp_ablkcipher_dec_test(struct cp_test *test)
 {
-	kcapi_aead_decrypt(test->u.aead.handle,
-			   test->u.aead.input,
-			   test->u.aead.indatalen,
-			   test->u.aead.iv,
-			   test->u.aead.output,
-			   test->u.aead.outdatalen,
-			   test->accesstype);
+	if (test->u.aead.aio)
+		kcapi_aead_decrypt_aio(test->u.aead.handle,
+				       test->u.aead.iniov,
+				       test->u.aead.outiov,
+				       test->u.aead.aio,
+				       test->u.aead.iv,
+				       test->accesstype);
+	else
+		kcapi_aead_decrypt(test->u.aead.handle,
+				   test->u.aead.input,
+				   test->u.aead.indatalen,
+				   test->u.aead.iv,
+				   test->u.aead.output,
+				   test->u.aead.outdatalen,
+				   test->accesstype);
 	return test->u.aead.outdatalen;
 }
 
