@@ -25,7 +25,8 @@
  * Synchronous symmetric ciphers
  ****************************************************************************/
 
-static int cp_skcipher_init_test(struct cp_test *test, size_t len)
+static int cp_skcipher_init_test(struct cp_test *test, size_t len,
+				 unsigned int aio)
 {
 	unsigned char *scratchpad = NULL;
 #define MAX_KEYLEN 64
@@ -41,7 +42,8 @@ static int cp_skcipher_init_test(struct cp_test *test, size_t len)
 		return -EFAULT;
 	}
 
-	if (kcapi_cipher_init(&test->u.skcipher.handle, test->driver_name, 0)) {
+	if (kcapi_cipher_init(&test->u.skcipher.handle, test->driver_name,
+			      aio ? KCAPI_INIT_AIO : 0)) {
 		printf(DRIVER_NAME": could not allocate skcipher handle for "
 		       "%s\n", test->driver_name);
 		goto out;
@@ -74,7 +76,8 @@ static int cp_skcipher_init_test(struct cp_test *test, size_t len)
 	test->u.skcipher.iv = ivdata;
 
 	err = posix_memalign((void *)&scratchpad, sysconf(_SC_PAGESIZE),
-		kcapi_cipher_blocksize(test->u.skcipher.handle) * len);
+		kcapi_cipher_blocksize(test->u.skcipher.handle) * len *
+				       (aio ? aio : 1));
 	if (err) {
 		printf(DRIVER_NAME": could not allocate scratchpad for "
 		       "%s (error %d)\n", test->driver_name, err);
@@ -82,12 +85,32 @@ static int cp_skcipher_init_test(struct cp_test *test, size_t len)
 	}
 
 	cp_read_random(scratchpad,
-		       kcapi_cipher_blocksize(test->u.skcipher.handle) * len);
+		       kcapi_cipher_blocksize(test->u.skcipher.handle) * len *
+					      (aio ? aio : 1));
 
 	test->u.skcipher.inputlen =
 		len * kcapi_cipher_blocksize(test->u.skcipher.handle);
 	test->u.skcipher.scratchpad = scratchpad;
 
+	if (aio) {
+		unsigned int i;
+
+		err = posix_memalign((void *)&test->u.skcipher.iovec, bs,
+				     aio * sizeof(struct iovec));
+		if (err) {
+			printf(DRIVER_NAME": could not allocate iovec buffer\n");
+			goto out;
+		}
+
+		for (i = 0; i < aio; i++) {
+			test->u.skcipher.iovec[i].iov_base = scratchpad;
+			test->u.skcipher.iovec[i].iov_len = test->u.skcipher.inputlen;
+			scratchpad += test->u.skcipher.inputlen;
+		}
+
+		test->u.skcipher.aio = aio;
+	}
+
 	return 0;
 
 out:
@@ -104,30 +127,48 @@ static void cp_skcipher_fini_test(struct cp_test *test)
 	dbg("Cleaning up asynchronous symmetric test %s\n", test->testname);
 	free(test->u.skcipher.scratchpad);
 	free(test->u.skcipher.iv);
+	if (test->u.skcipher.aio)
+		free(test->u.skcipher.iovec);
 	kcapi_cipher_destroy(test->u.skcipher.handle);
 }
 
-static unsigned int cp_ablkcipher_enc_test(struct cp_test *test)
+static unsigned int cp_skcipher_enc_test(struct cp_test *test)
 {
-	kcapi_cipher_encrypt(test->u.skcipher.handle,
-			     test->u.skcipher.scratchpad,
-			     test->u.skcipher.inputlen,
-			     test->u.skcipher.iv,
-			     test->u.skcipher.scratchpad,
-			     test->u.skcipher.inputlen,
-			     test->accesstype);
+	if (test->u.skcipher.aio)
+		kcapi_cipher_encrypt_aio(test->u.skcipher.handle,
+					 test->u.skcipher.iovec,
+					 test->u.skcipher.iovec,
+					 test->u.skcipher.aio,
+					 test->u.skcipher.iv,
+					 test->accesstype);
+	else
+		kcapi_cipher_encrypt(test->u.skcipher.handle,
+				     test->u.skcipher.scratchpad,
+				     test->u.skcipher.inputlen,
+				     test->u.skcipher.iv,
+				     test->u.skcipher.scratchpad,
+				     test->u.skcipher.inputlen,
+				     test->accesstype);
 	return test->u.skcipher.inputlen;
 }
 
-static unsigned int cp_ablkcipher_dec_test(struct cp_test *test)
+static unsigned int cp_skcipher_dec_test(struct cp_test *test)
 {
-	kcapi_cipher_decrypt(test->u.skcipher.handle,
-			     test->u.skcipher.scratchpad,
-			     test->u.skcipher.inputlen,
-			     test->u.skcipher.iv,
-			     test->u.skcipher.scratchpad,
-			     test->u.skcipher.inputlen,
-			     test->accesstype);
+	if (test->u.skcipher.aio)
+		kcapi_cipher_decrypt_aio(test->u.skcipher.handle,
+					 test->u.skcipher.iovec,
+					 test->u.skcipher.iovec,
+					 test->u.skcipher.aio,
+					 test->u.skcipher.iv,
+					 test->accesstype);
+	else
+		kcapi_cipher_decrypt(test->u.skcipher.handle,
+				     test->u.skcipher.scratchpad,
+				     test->u.skcipher.inputlen,
+				     test->u.skcipher.iv,
+				     test->u.skcipher.scratchpad,
+				     test->u.skcipher.inputlen,
+				     test->accesstype);
 	return test->u.skcipher.inputlen;
 }
 
@@ -151,10 +192,15 @@ static const struct cp_skcipher_tests testcases[] = {
 	{ "AES(i586) CBC(G) 128", "cbc(aes-asm)", 16 },
 	{ "AES(i586) CBC(G) 192", "cbc(aes-asm)", 24 },
 	{ "AES(i586) CBC(G) 256", "cbc(aes-asm)", 32 },
+
 	{ "AES(MV-CESA) CBC(MV-CESA) 128", "mv-cbc-aes", 16 },
 	{ "AES(MV-CESA) CBC(MV-CESA) 192", "mv-cbc-aes", 24 },
 	{ "AES(MV-CESA) CBC(MV-CESA) 256", "mv-cbc-aes", 32 },
 
+	{ "AES(hisi) CBC 128", "hisi_sec_aes_cbc", 16 },
+	{ "AES(hisi) CBC 192", "hisi_sec_aes_cbc", 24 },
+	{ "AES(hisi) CBC 256", "hisi_sec_aes_cbc", 32 },
+
 	{ "AES(G) CTR(G) 128", "ctr(aes-generic)", 16 },
 	{ "AES(G) CTR(G) 192", "ctr(aes-generic)", 24 },
 	{ "AES(G) CTR(G) 256", "ctr(aes-generic)", 32 },
@@ -422,9 +468,9 @@ void cp_skcipher_register(struct cp_test **skcipher_test, size_t *entries)
 			cp_skcipher_testdef[j].init_test = cp_skcipher_init_test;
 			cp_skcipher_testdef[j].fini_test = cp_skcipher_fini_test;
 			if (enc)
-				cp_skcipher_testdef[j].exec_test = cp_ablkcipher_enc_test;
+				cp_skcipher_testdef[j].exec_test = cp_skcipher_enc_test;
 			else
-				cp_skcipher_testdef[j].exec_test = cp_ablkcipher_dec_test;
+				cp_skcipher_testdef[j].exec_test = cp_skcipher_dec_test;
 		}
 	}
 	*skcipher_test = &cp_skcipher_testdef[0];
