@@ -71,6 +71,8 @@ struct skcipher_def {
 	unsigned char *scratchpad;
 	unsigned char *iv;
 	size_t inputlen;
+	unsigned int aio;
+	struct iovec *iovec;
 	struct kcapi_handle *handle;
 };
 
@@ -82,6 +84,9 @@ struct aead_def {
 	size_t indatalen;
 	size_t outdatalen;
 	size_t assoclen;
+	unsigned int aio;
+	struct iovec *iniov;
+	struct iovec *outiov;
 	struct kcapi_handle *handle;
 };
 
@@ -108,7 +113,7 @@ struct cp_test {
 	int enc;
 	unsigned int exectime;
 	struct cp_res results;
-	int (*init_test)(struct cp_test *test, size_t len);
+	int (*init_test)(struct cp_test *test, size_t len, unsigned int aio);
 	unsigned int (*exec_test)(struct cp_test *test);
 	void (*fini_test)(struct cp_test *test);
 
@@ -171,7 +176,8 @@ static inline void cp_zfree(void *ptr, unsigned int len)
  * general functions
  */
 char *cp_print_status(struct cp_test *test, int raw);
-int cp_exec_test(struct cp_test *test, unsigned int exectime, size_t len);
+int cp_exec_test(struct cp_test *test, unsigned int exectime, size_t len,
+		 unsigned int aio);
 int cp_read_random(unsigned char *buf, size_t buflen);
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
